/*
  Program........: JDBCDisplayQueryJDK12.java
  Original Code..: Base code was taken from Deitel & Deitel's "Java, How to Program" 3rd Edition.
                   Chapter 12, pg 913 Fig 18.28 "DisplayQueryResults.java"
  Modification by: Michael Thomas
    * Allow the passing of database information via the DOS Prompt.
    * Now        - Using "appletviewer" this program will run as an JApplet or application.
    * For future - Using Browser Netscape 6 (also: Java Plugin or Browser supporting JDK 1.2) 
    *              with properities file (Security file) this program should work.

  Notes:
  
  * Errors - If you see "Type was: ...." then you need to edit the case statement
  *          to contain the type that is displayed.

*/
// 
import java.sql.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

//####################################################################################
public class JDBCDisplayQueryJDK12 extends JApplet {
//####################################################################################
  
   // java.sql types needed for database processing
   private Connection connection;
   private Statement statement;
   private ResultSet resultSet;
   private ResultSetMetaData rsMetaData;

   // javax.swing types needed for GUI
   private JTable table;
   private JTextArea inputQuery;
   private JButton submitQuery;

   private String strDbDriver   = "sun.jdbc.odbc.JdbcOdbcDriver"; //Default DB Driver
   private String strDbUrl      = "jdbc:odbc:JDBCDisplayQuery_access2000"; //Default DB URL
   private String strDbUserName = "guest";              //Default Username
   private String strDbPassword = "guest";              //Default Password
   private String strDbTable    = "Test";               //Default Table

   //------------------------------------------------------------------------------- 
   public void init()  {   
   //------------------------------------------------------------------------------- 
     
      //super( "Enter Query. Click Submit to See Results." );

      // The URL specifying the Books database to which
      // this program connects using JDBC to connect to a
      // Microsoft ODBC database.
      strDbUrl      = strDbUrl;
      strDbUserName = strDbUserName;
      strDbPassword = strDbPassword;

      // Load the driver to allow connection to the database
      try {
         Class.forName( strDbDriver );

         connection = DriverManager.getConnection( 
            strDbUrl, strDbUserName, strDbPassword );
      } catch ( ClassNotFoundException cnfex ) {
         System.err.println( 
            "Failed to load JDBC/ODBC driver." );
         cnfex.printStackTrace();
         System.exit( 1 );  // terminate program
      } catch ( SQLException sqlex ) {
         System.err.println( "Unable to connect" );
         sqlex.printStackTrace();
         System.exit( 1 );  // terminate program
      }

      // If connected to database, set up GUI      
      inputQuery = new JTextArea( "SELECT * FROM " + strDbTable, 4, 30 );
      submitQuery = new JButton( "Submit query" );
      submitQuery.addActionListener(
         new ActionListener() {
            public void actionPerformed( ActionEvent e ) 
            {
               getTable();
            }
         }   
      );

      JPanel topPanel = new JPanel();
      topPanel.setLayout( new BorderLayout() );
      topPanel.add( new JScrollPane( inputQuery), 
                    BorderLayout.CENTER );
      topPanel.add( submitQuery, BorderLayout.SOUTH );
 
      table = new JTable( 4, 4 );

      Container c = getContentPane();
      c.setLayout( new BorderLayout() );
      c.add( topPanel, BorderLayout.NORTH );
      c.add( table, BorderLayout.CENTER );      //Adds a blank Table.

      getTable();

      setSize( 500, 500 );
      //show(); //Deprecated
      setVisible( true );
   }
  //------------------------------------------------------------------------------- 
   private void getTable() {
  //------------------------------------------------------------------------------- 
     
      try {
         String query = inputQuery.getText();

         statement = connection.createStatement();
         resultSet = statement.executeQuery( query );         
         displayResultSet( resultSet );
      }  catch ( SQLException sqlex ) {
         sqlex.printStackTrace();
      }
      
   }
  //------------------------------------------------------------------------------- 
   private void displayResultSet( ResultSet rs ) throws SQLException   {
  //------------------------------------------------------------------------------- 
     
      // position to first record
      boolean moreRecords = rs.next();  

      // If there are no records, display a message
      if ( ! moreRecords ) {
         JOptionPane.showMessageDialog( this, 
            "ResultSet contained no records" );
         //setTitle( "No records to display" ); //zzz fix later
         return;
      }

      Vector columnHeads = new Vector();
      Vector rows = new Vector();

      try {
         // get column heads
         ResultSetMetaData rsmd = rs.getMetaData();
      
         for ( int i = 1; i <= rsmd.getColumnCount(); ++i ) 
            columnHeads.addElement( rsmd.getColumnName( i ) );

         // get row data
         do {
            rows.addElement( getNextRow( rs, rsmd ) ); 
         } while ( rs.next() );

         // display table with ResultSet contents
         
         table = new JTable( rows, columnHeads );
         
         JScrollPane scroller = new JScrollPane( table );

         Container c = getContentPane();
         c.remove( 1 );
         c.add( scroller, BorderLayout.CENTER );
         c.validate();
      } catch ( SQLException sqlex ) {
         sqlex.printStackTrace();
      }
   }
  //------------------------------------------------------------------------------- 
   private Vector getNextRow( ResultSet rs, 
                              ResultSetMetaData rsmd )
       throws SQLException   {
  //------------------------------------------------------------------------------- 
         
      Vector currentRow = new Vector();
      
      for ( int i = 1; i <= rsmd.getColumnCount(); ++i )
         switch( rsmd.getColumnType( i ) ) {
            case Types.VARCHAR:
            case Types.LONGVARCHAR:
                  currentRow.addElement( rs.getString( i ) );
               break;
            case Types.INTEGER:
                  currentRow.addElement( 
                     new Long( rs.getLong( i ) ) );
               break;
            case Types.CHAR:  //Added by Michael Thomas
                  currentRow.addElement( rs.getString( i ) );
               break;
            default: 
               System.out.println( "Type was: " + 
                  rsmd.getColumnTypeName( i ) );
         }
      
      return currentRow;
   }
  //------------------------------------------------------------------------------- 
   public void shutDown() {
  //------------------------------------------------------------------------------- 
      try {
         connection.close();
      } catch ( SQLException sqlex ) {
         System.err.println( "Unable to disconnect" );
         sqlex.printStackTrace();
      }
   }
  //------------------------------------------------------------------------------- 
  public static void main( String args [] ) { 
  //------------------------------------------------------------------------------- 
    
    JFrame app = new JFrame( "JDBCDisplayQueryJDK12.java" );  
    app.setSize( 500, 500);                         //set Frame: width, height

    final JDBCDisplayQueryJDK12 japplet = new JDBCDisplayQueryJDK12();         //create the applet.
 
    if ( args.length == 5 ) {  //Load command line overrides.
      japplet.strDbDriver   = args[0]; 
      japplet.strDbUrl      = args[1];
      japplet.strDbUserName = args[2];
      japplet.strDbPassword = args[3];
      japplet.strDbTable    = args[4];
    }

    app.addWindowListener( 
         new WindowAdapter() {
            public void windowClosing( WindowEvent e ) 
            {  
               japplet.shutDown();
               System.exit( 0 );
            }
         }
    );

    app.getContentPane().add( japplet, BorderLayout.CENTER );       //add applet to center of frame.
    japplet.init();                                //initialize applet.
    japplet.start();                               //start applet.
    app.setVisible( true );                    //Make frame visible.

  } //main()
  
  //#############################################################################
  class CustAwtTable extends ScrollPane {
  //#############################################################################
    Vector avecCurrentRow;
  //------------------------------------------------------------------------------- 
    public CustAwtTable ( Vector avecRows, Vector avecHeadings ) {
  //------------------------------------------------------------------------------- 
      
      int intRow    = avecRows.size();
      int intColumn = avecHeadings.size();
      Label lblText;
      
      Panel pnlTable = new Panel();
      GridBagLayout gblLayout = new GridBagLayout();
      GridBagConstraints gbcConstraints = new GridBagConstraints();
      
      pnlTable.setLayout( gblLayout );
      
      gbcConstraints.weightx = 1.0;
      gbcConstraints.weighty = 0.0;
      gbcConstraints.gridx = GridBagConstraints.RELATIVE;  //Default is RELATIVE
      gbcConstraints.gridy = GridBagConstraints.RELATIVE;  //Default is RELATIVE
      gbcConstraints.gridwidth = 1;
      gbcConstraints.gridheight = 1;
      gbcConstraints.anchor = GridBagConstraints.NORTHWEST;
            
      for ( int x = 0; x < intColumn; x++ ) {
        lblText = new Label ( (String) avecHeadings.get(x) );
        if ( x == intColumn -1 ) {
          gbcConstraints.gridwidth = GridBagConstraints.REMAINDER ; //End of Row
        }
        gblLayout.setConstraints( lblText, gbcConstraints);
        pnlTable.add( lblText );
      }
      
      for ( int x = 0; x < intRow; x++ ) {
        gbcConstraints.gridwidth = 1;  //Reset the gridwidth        
        avecCurrentRow = (Vector) avecRows.get(x);
        int intCurrentSize = avecCurrentRow.size();
        for ( int intCol = 0; intCol < intCurrentSize ; intCol++ ) {
          lblText = new Label ( (String) ""+avecCurrentRow.get(intCol) );
          if ( intCol == intCurrentSize -1 ) {
            gbcConstraints.gridwidth = GridBagConstraints.REMAINDER ; //End of Row
          }
          if ( x == intRow -1 ) {
            gbcConstraints.gridheight = GridBagConstraints.REMAINDER ; //End of Table
          }
          gblLayout.setConstraints( lblText, gbcConstraints);          
          pnlTable.add( lblText );
        }
      }      
      
      this.add( pnlTable );
    }
    
  } //eoc

} //eof